diff --git a/lists.lisp b/lists.lisp
index 00b42fa..d44e27d 100644
--- a/lists.lisp
+++ b/lists.lisp
@@ -278,8 +278,38 @@ not destructively modified. Keys are compared using EQ."
 (defun delete-from-plist (plist &rest keys)
   "Just like REMOVE-FROM-PLIST, but this version may destructively modify the
 provided plist."
-  ;; FIXME: should not cons
-  (apply 'remove-from-plist plist keys))
+  (labels ((assert-proper-plist (x)
+             (assert x () "Expected a proper plist, got ~S" plist))
+           (bad-key-p (key)
+             (member key keys :test #'eq))
+           (find-first ()
+             "Find the first cons in PLIST to keep."
+             (loop :for the-cons :on plist :by #'cddr
+                   :unless (prog1 (bad-key-p (car the-cons))
+                             (assert-proper-plist (cdr the-cons)))
+                     :do (return the-cons)
+                   :finally (return nil))))
+    (declare (inline assert-proper-plist
+                     bad-key-p
+                     find-first))
+    ;; Find the first good key and delete any bad key-value pairs
+    ;; between it and the start.
+    (let ((first (find-first)))
+      (unless (eq first plist)
+        (setf (cddr plist)
+              first))
+      
+      ;; At this point, we know FIRST points to the first key
+      ;; which exists, or NIL.
+      (loop :with last-good := first    ; Keep the last good key
+            :for the-cons :on (cddr first) :by #'cddr
+            :do (progn
+                  (assert-proper-plist (cdr the-cons))
+                  (if (bad-key-p (car the-cons))
+                      (setf (cddr last-good)
+                            (cddr the-cons))
+                      (setf last-good the-cons)))
+            :finally (return first)))))
 
 (define-modify-macro remove-from-plistf (&rest keys) remove-from-plist
                      "Modify macro for REMOVE-FROM-PLIST.")
